from awl import AstSerialization


def test_ast_generation():
    ast_serialization = AstSerialization()
    source = """if a == 1:
    b = 1
else:
    b = 'test'"""

    ast_dict = ast_serialization.parse(source)
    # print(json.dumps(ast_dict, indent=4))
    print(ast_serialization.dumps())
    assert ast_dict == {
        "_type": "Module",
        "body": [
            {
                "_type": "If",
                "body": [
                    {
                        "_type": "Assign",
                        "targets": [{"_type": "Name", "id": "b"}],
                        "value": {"_type": "Constant", "value": 1},
                    }
                ],
                "orelse": [
                    {
                        "_type": "Assign",
                        "targets": [{"_type": "Name", "id": "b"}],
                        "value": {"_type": "Constant", "value": "test"},
                    }
                ],
                "test": {
                    "_type": "Compare",
                    "comparators": [{"_type": "Constant", "value": 1}],
                    "left": {"_type": "Name", "id": "a"},
                    "ops": [{"_type": "Eq"}],
                },
            }
        ],
        "type_ignores": [],
    }

    src_code = ast_serialization.unparse(ast_dict)
    # print(src_code)
    assert src_code == source

    # manipulate ast_dict: set b = 2
    ast_dict["body"][0]["body"][0]["value"]["value"] = 2
    src_code = ast_serialization.unparse(ast_dict)
    # print(src_code)
    assert (
        src_code
        == """if a == 1:
    b = 2
else:
    b = 'test'"""
    )


def test_rdf_generation():
    ast_serialization = AstSerialization()
    source = """if a == 1:
    b = 1
else:
    b = 'test'"""

    ast_serialization.parse(source)
    jsonld_doc = ast_serialization.to_jsonld()

    # import into a graph
    from rdflib import Graph

    g = Graph()
    g.parse(data=jsonld_doc, format="json-ld")

    # dump graph as turtle
    print(g.serialize(format="turtle"))

    # query for all possible values of b
    qres = g.query(
        """
        PREFIX awl: <https://w3id.org/awl/schema/>
        PREFIX ex: <https://example.org/>
        SELECT ?v
        WHERE {
            ?a a awl:Assign .
            ?a awl:HasTarget ex:b .
            ?a awl:HasValue ?v .
        }
        """
    )

    expected_possible_values = [1, "test"]
    possible_values = [row[0].toPython() for row in qres]
    print(possible_values)
    assert possible_values == expected_possible_values


if __name__ == "__main__":
    test_ast_generation()
    test_rdf_generation()



import ast
import json

import yaml
from ast2json import ast2json
from json2ast import json2ast

from awl import jsonld_context


class AstSerialization:
    def __init__(self):
        pass

    @staticmethod
    def del_keys(d: dict, keys: list) -> dict:
        for key in keys:
            if key in d:
                del d[key]
        for key, value in d.items():
            if isinstance(value, dict):
                AstSerialization.del_keys(value, keys)
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):
                        AstSerialization.del_keys(item, keys)
        return d

    @staticmethod
    def add_key(d: dict, k: str, v) -> dict:
        d[k] = v
        for key, value in d.items():
            if isinstance(value, dict):
                AstSerialization.add_key(value, k, v)
            elif isinstance(value, list):
                for item in value:
                    if isinstance(item, dict):
                        AstSerialization.add_key(item, k, v)
        return d

    def parse(self, source: str) -> dict:
        ast_dict = ast2json(ast.parse(source))

        rm_keywords = [
            "col_offset",
            "end_col_offset",
            "end_lineno",
            "lineno",
            "type_comment",
            "n",
            "s",
            "kind",
            "ctx",
        ]

        ast_dict = self.del_keys(ast_dict, rm_keywords)  # remove annotations
        self.ast_dict = ast_dict
        return ast_dict

    def unparse(self, ast_dict: dict = None) -> str:
        ast_dict = self.add_key(ast_dict, "lineno", 0)  # needed to unparse
        ast_tree = json2ast(ast_dict)
        source = ast.unparse(ast_tree)
        return source

    def dumps(self, format="yaml") -> str:
        res = ""
        if format == "json":
            res = json.dumps(self.ast_dict, indent=4)
        elif format == "yaml":
            res = yaml.dump(self.ast_dict, indent=4)
        return res

    def to_jsonld(self) -> dict:
        res = {"@context": jsonld_context.awl_context["@context"], **self.ast_dict}
        return res



awl_context = {
    "@context": [
        {
            "awl": "https://w3id.org/awl/schema/",
            "ex": "https://example.org/",
            "@base": "https://w3id.org/awl/schema/",
            "_type": "@type",
            "id": "@id",
            "body": "awl:HasPart",
            "Name": {
                "@id": "awl:Variable",
                "@context": {"@base": "https://example.org/"},
            },
            "targets": "awl:HasTarget",
            "value": {"@id": "awl:HasValue", "@context": {"value": "@value"}},
            "If": {"@id": "awl:If", "@context": {"body": "awl:IfTrue"}},
            "orelse": "awl:IfFalse",
            "test": "awl:HasCondition",
            "comparators": "awl:HasRightHandComparator",
            "ops": "awl:HasOperator",
            "left": "awl:HasLeftHandComparator",
            "func": {
                "@id": "awl:HasFunctionCall",
                "@context": {
                    "@base": "https://example.org/",
                    "Name": "awl:Function",
                    "value": "awl:HasValue",
                },
            },
            "args": "awl:HasArgument",
            "keywords": {
                "@id": "awl:HasKeywordArgument",
                "@context": {"value": "awl:HasValue"},
            },
            "arg": "awl:HasKey",
        }
    ]
}
